package org.jboss.as.integration.hornetq.deployers;

import java.io.InputStream;

import org.hornetq.core.config.Configuration;
import org.hornetq.core.deployers.impl.FileConfigurationParser;
import org.jboss.deployers.vfs.spi.deployer.AbstractVFSParsingDeployer;
import org.jboss.deployers.vfs.spi.structure.VFSDeploymentUnit;
import org.jboss.logging.Logger;
import org.jboss.vfs.VirtualFile;

/**
 * This Deployer will take a hornetq-configuration.xml parse the Configuration
 * and attach the Configuration output into the deployment unit. Another
 * deployer that takes Configuration as an input will then take over and install
 * the queues at the proper places.
 * 
 * @see HornetQJMSRealDeployer
 * 
 * @author <mailto:clebert.suconic@jboss.org">Clebert Suconic</a>
 * 
 */
public class HornetQConfigParserDeployer extends AbstractVFSParsingDeployer<Configuration>
{

   private static final Logger log = Logger.getLogger(HornetQConfigParserDeployer.class);

   private final FileConfigurationParser parser;

   public HornetQConfigParserDeployer(final String name)
   {
      super(Configuration.class);

      this.parser = new FileConfigurationParser();
      
      this.setSuffix(name);
   }
   
   

   @Override
   protected Configuration parse(VFSDeploymentUnit unit, VirtualFile file, Configuration root) throws Exception
   {

      InputStream deploymentStream = null;

      try
      {
         deploymentStream = file.openStream();
         Configuration config = parser.parseMainConfig(deploymentStream);
        
         if (log.isTraceEnabled())
         {
            log.trace("Parser parsed Main Config = " + config);
         }

         return config;
      } finally
      {
         try
         {
            deploymentStream.close();
         } catch (Exception ignored)
         {
         }
      }
   }
}
